package visitor_demo.book;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Reads a set of files representing the chapters of a book and builds a Book
 * object out of it.
 * 
 * @author Krysta Yousoufian, CSE 331
 * 
 */
public class BookBuilder {

    /**
     * Loads a book.
     * 
     * @param dirName
     *            Path to a directory whose name represents the book title with
     *            underscores in place of spaces. Directory contains files with
     *            names 1,2,...,n containing the contents of the n chapters in
     *            the book.
     * @return A book built from the contents of the given directory.
     */
    public static Book buildBook(String dirName) {
        File bookDir = new File(dirName);
        String title = bookDir.getName().replace("_", " ");

        // Construct each chapter from the corresponding file.
        File[] chapterFiles = bookDir.listFiles();
        Chapter[] chapters = new Chapter[chapterFiles.length];
        for (File chapterFile : chapterFiles) {
            int idx = Integer.parseInt(chapterFile.getName());

            try {
                chapters[idx - 1] = buildChapter(chapterFile);
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }
        }

        // Convert Chapter[] to List<Chapter>.
        List<Chapter> chapterList = new ArrayList<Chapter>();
        for (Chapter c : chapters)
            chapterList.add(c);

        return new Book(title, chapterList);
    }

    /**
     * Loads a chapter of a book.
     * 
     * @param chapterFile
     *            The file containing the contents of the given chapter.
     * @return A Chapter built from the given file.
     * @throws IOException
     *             If file cannot be opened or read successfully.
     */
    private static Chapter buildChapter(File chapterFile) throws IOException {
        BufferedReader rdr = new BufferedReader(new FileReader(chapterFile));
        String title = rdr.readLine();
        // Skip blank line after title.
        rdr.readLine();

        // Construct each paragraph.
        List<Paragraph> paragraphs = new ArrayList<Paragraph>();
        Paragraph pgph = buildParagraph(rdr);
        while (pgph != null) {
            paragraphs.add(pgph);
            pgph = buildParagraph(rdr);
        }
        return new Chapter(title, paragraphs);
    }

    /**
     * Loads a paragraph of a book by reading text from the given rdr until a
     * blank line or end of file is reached.
     * 
     * @param rdr
     *            A reader for a file containing the given paragraph, initially
     *            pointing at the start of the paragraph
     * @return A paragraph built from the rdr.
     * @effects Moves the rdr to point just past the next blank line, or to the
     *          endof the file if there are no more blank lines.
     * @throws IOException
     *             If file cannot be opened or read successfully.
     */
    private static Paragraph buildParagraph(BufferedReader rdr)
            throws IOException {
        String line = rdr.readLine();
        if (line == null)
            return null;

        // Add text to the paragraph until we hit a blank line or end of file.
        StringBuilder textBuilder = new StringBuilder();
        while (line != null && !line.isEmpty()) {
            textBuilder.append(line + "\n");
            line = rdr.readLine();
        }

        return new Paragraph(textBuilder.toString().trim());
    }
}
